import json
from datetime import datetime

from wagtail.core.models import Page, Site

import pytest
from taggit.models import Tag

from pfmsite.blog.models import ArticlePage, BlogPage
from pfmsite.conftest import tagged_podcasts  # noqa: F401
from pfmsite.core.models import HomePage


@pytest.fixture
def home_page(tagged_podcasts): # noqa F811
    """
    Création d'une page d'accueil, que l'on ajoute au début de l'arborescence
    ainsi qu'au site
    """
    site = Site.objects.get(is_default_site=True)
    site.root_page.delete()
    root = Page.get_first_root_node()
    tags = Tag.objects.all()
    home = HomePage(
        title="Titre de la Home",
        tags=json.dumps([{'type': 'tag', 'value': tag.id} for tag in tags]),
    )
    root.add_child(instance=home)
    site.root_page = home
    site.save()
    return home


@pytest.fixture
def blog_page(home_page):
    blog_page_index = BlogPage(title="Liste des articles d'actualité")
    home_page.add_child(instance=blog_page_index)
    return blog_page_index


@pytest.fixture
def article_page(blog_page):
    article_page = ArticlePage(
        title="Une actualité musicale",
        introduction="Voici l'introduction de l'actualité",
        first_published_at=datetime(2022, 1, 7),
    )
    article_page.tags.add("Musique")
    blog_page.add_child(instance=article_page)
    return article_page
