from django.urls import reverse

from wagtail.tests.utils.form_data import rich_text, streamfield

import pytest
from bs4 import BeautifulSoup

from pfmsite.utils.tests import ViewMixin, get_streamfield_form_data

from ..models import StandardPage


@pytest.mark.django_db
class TestStandardPage(ViewMixin):
    model = StandardPage

    def test_admin_add(self, admin_user, home_page):
        url = reverse(
            'wagtailadmin_pages:add',
            args=(
                self.model._meta.app_label,
                self.model._meta.model_name,
                home_page.pk,
            ),
        )

        form = self.get(url, user=admin_user).forms[1]
        form['title'] = "À propos"
        form['slug'] = "a-propos"
        extra_fields = get_streamfield_form_data(
            'body',
            [
                streamfield(
                    [
                        (
                            'paragraph_block',
                            rich_text("<p>Une <b>description</b>.</p>"),
                        )
                    ]
                ),
            ],
        )
        self.submit(form, extra_fields, status=302).follow()

        page = self.model.objects.get(title="À propos")
        assert page.show_in_menus is True


@pytest.mark.django_db
class TestHomePage:
    tags_names = ['Musique', 'Social', 'Culture']

    def get_cols(self, soup):
        cols = soup.select('.main-content .col')
        content_col = {}
        for i, tag_name in enumerate(self.tags_names):
            content_col[tag_name] = [
                c.text for c in cols[i].select('.card-title')
            ]
        return content_col

    def test_show_tagged_contents(
        self, client, home_page, article_page, tagged_podcasts
    ):

        resp = client.get('/')
        soup = BeautifulSoup(resp.content, features="lxml")

        tag_titles = [t.text for t in soup.select('.col .title')]
        assert self.tags_names == tag_titles

        content_col = self.get_cols(soup)

        assert article_page.title in content_col["Musique"]

        assert tagged_podcasts[0].content_object.title in content_col["Musique"]
        assert tagged_podcasts[1].content_object.title in content_col["Social"]
        assert tagged_podcasts[2].content_object.title in content_col["Culture"]

    def test_items_contents_order(
        self, client, home_page, article_page, tagged_podcasts
    ):
        resp = client.get('/')
        soup = BeautifulSoup(resp.content, features="lxml")
        musique_col = self.get_cols(soup)['Musique']
        assert musique_col == ['Podcast #7', 'Une actualité musicale']
