from django.views.generic import TemplateView

from wagtail.search.backends import get_search_backend
from wagtail.search.models import Query

from wagtail_webradio.models import Podcast

from pfmsite.blog.models import ArticlePage
from pfmsite.core.models import StandardPage


class SearchView(TemplateView):

    template_name = 'core/search_results.html'

    def get(self, request, *args, **kwargs):
        self.query = self.request.GET.get('q', None)
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.query:
            context['standard_page_results'] = (
                StandardPage.objects.live().public().search(self.query)
            )
            context['articles_results'] = (
                ArticlePage.objects.live().public().search(self.query)
            )
            search_backend = get_search_backend()
            context['podcasts_results'] = search_backend.search(
                self.query,
                Podcast.objects.all(),
            )
            # ajoute un score pour de potentiels résultats favorisés
            Query.get(self.query).add_hit()
        context['search_query'] = self.query
        return context
