from django.urls import reverse

import pytest
from bs4 import BeautifulSoup


def get_soup(content):
    return BeautifulSoup(content, 'html.parser')


@pytest.mark.django_db
class TestPodcastListView:
    def test_status_code(self, client, podcast):
        assert client.get(reverse('webradio:podcast_list')).status_code == 200

    def test_filter_podcast_by_tag(self, client, podcast, one_tagged_podcast):
        response = client.get(
            '{}?tag={}'.format(
                reverse('webradio:podcast_list'),
                'music',
            )
        )
        cards = get_soup(response.content).select('.card')
        assert len(cards) == 1
        assert 'Dansons' in cards[0].get_text()
        assert 'interview' not in cards[0].get_text()


@pytest.mark.django_db
class TestRadioShowListView:
    def test_status_code(self, client):
        assert client.get(reverse('webradio:radioshow_list')).status_code == 200

    def test_no_radioshow(self, client):
        resp = client.get(reverse('webradio:radioshow_list'))
        assert get_soup(resp.content).select('.card') == []

    def test_radioshow_display(self, client, radio_show):
        resp = client.get(reverse('webradio:radioshow_list'))
        assert len(get_soup(resp.content).select('.card')) == 1


@pytest.mark.django_db
class TestRadioShowDetailView:
    def test_status_code(self, client, radio_show):
        resp = client.get(
            reverse('webradio:radioshow_detail', args=[radio_show.pk])
        )
        assert resp.status_code == 200

    def test_podcasts_displayed(self, client, radio_show, podcast):
        resp = client.get(
            reverse('webradio:radioshow_detail', args=[radio_show.pk])
        )
        title = get_soup(resp.content).select('.card .card-title')[0].text
        assert podcast.title == title
