from django.http import Http404
from django.views.generic import DetailView, ListView

from django_filters.views import FilterView
from taggit.models import Tag
from wagtail_webradio.models import Podcast, RadioShow

from .filters import PodcastFilter


class PodcastDetailView(DetailView):
    model = Podcast
    template_name = 'wagtailwebradio/podcast_detail.html'

    def get_object(self):
        pk = self.kwargs.get(self.pk_url_kwarg)
        try:
            obj = Podcast.objects.currents().get(pk=pk)
        except Podcast.model.DoesNotExist:
            raise Http404('Podcast not found')
        return obj


class PodcastListView(FilterView):
    """Podcast list view"""

    model = Podcast
    template_name = 'wagtailwebradio/podcast_list.html'
    filterset_class = PodcastFilter
    paginate_by = 30

    def get_queryset(self):
        queryset = self.model.objects.currents()

        # filtre par tag
        tag = self.request.GET.get('tag')
        if tag:
            queryset = queryset.filter(tags__name=tag)
        return queryset

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        radio_show_pk = self.request.GET.get('radio_show')
        if radio_show_pk:
            context['radio_show'] = RadioShow.objects.get(pk=radio_show_pk)
        context['tags'] = Tag.objects.order_by('name')
        return context


class RadioShowListView(ListView):
    model = RadioShow
    template_name = 'wagtailwebradio/radioshow_list.html'
    queryset = RadioShow.objects.order_by('title')


class RadioShowDetailView(DetailView):
    model = RadioShow
    template_name = 'wagtailwebradio/radioshow_detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['last_podcasts'] = Podcast.objects.filter(
            radio_show=self.get_object()
        ).order_by('-publish_date')[:6]
        return context
