from wagtail.tests.utils.form_data import nested_form_data, streamfield

import pytest


def get_streamfield_form_data(name, items):
    """
    Retourne les données de forumulaire d'un champ de type StreamField
    nommé `name` pour les blocs `items`, donnés sous forme d'une liste
    (block_type, value).
    """
    # https://docs.wagtail.io/en/stable/advanced_topics/testing.html#wagtail.tests.utils.form_data.streamfield
    return nested_form_data({name: streamfield(items)})


class ViewMixin:
    """
    Mixin ajoutant des facilitées pour faire des requêtes avec Webtest, à
    appliquer aux classes définissant des tests sur les vues.
    """

    #: L'URL à utiliser par défaut pour les requêtes.
    url = None

    #: L'utilisateur à positionner par défaut dans les requêtes.
    user = None

    #: Détermine s'il faut activer la protection CSRF.
    csrf_checks = True

    @pytest.fixture(autouse=True)
    def setup_django_app(self, django_app_factory):
        self.django_app = django_app_factory(csrf_checks=self.csrf_checks)

    def _query(self, method, url=None, **kwargs):
        """
        Exécute la requête de type `method` vers `url`. Pour les autres
        arguments disponibles, voir `webtest.app.TestApp.get`.
        """
        kwargs.setdefault('user', self.user)
        return getattr(self.django_app, method)(url or self.url, **kwargs)

    def get(self, url=None, **kwargs):
        return self._query('get', url, **kwargs)

    def post(self, url=None, **kwargs):
        return self._query('post', url, **kwargs)

    def submit(
        self, form, extra_fields, name=None, index=None, value=None, **args
    ):
        """
        Étends `webtest.forms.Form.submit()` afin de pouvoir envoyer des
        champs supplémentaires définis par `extra_fields`.
        """
        fields = form.submit_fields(name, index=index, submit_value=value)
        if isinstance(extra_fields, dict):
            extra_fields = [(n, v) for n, v in extra_fields.items()]
        fields.extend(extra_fields)
        if form.method.upper() != 'GET':
            args.setdefault('content_type', form.enctype)
        response = form.response
        extra_environ = args.setdefault('extra_environ', {})
        extra_environ.setdefault('HTTP_REFERER', str(response.request.url))
        return response.goto(
            form.action, method=form.method, params=fields, **args
        )
